C***********************************************************************
C*                       SKYDOSE (version 2.3b)                        *
C*                                                                     *
C*  This program evaluates the gamma-ray skyshine dose from an         *
C*  isotropic, polyenergetic, point gamma-photon source collimated by  *
C*  three simple geometries: an open silo, a vertical black wall, and  *
C*  a rectangular building.  In all three geometries an optional       *
C*  overhead slab shield may be specified.                             *
C*                                                                     *
C*  The calculation is based on the integral line-beam method using    *
C*  an improved 3-parameter approximation for the line-beam response   *
C*  function.  For shielded sources, the method of exponential         *
C*  attenuation with buildup is used.                                  *
C*                                                                     *
C*  Source energy E must be  0.02 <= E <= 100 MeV, except for sources  *
C*  with an overhead shield.  For the shielded case, E <= 10 MeV.      *
C*                                                                     *
C*  References for the calculational methods are:                      *
C*  (1) J.K. Shultis, R.E. Faw and M.S. Bassett, "The Integral Line-   *
C*      Beam Method for Gamma Skyshine Analysis," Nucl. Sci. Engg.,    *
C*      107, 228-245 (1991).                                           *
C*  (2) J.K. Shultis, and R.E. Faw, "Extensions to the Integral Line-  *
C*      Beam Method for Gamma-Ray Skyshine Analyses," SAND-xxxx,       *
C*      Sandia National Lab., Albuquerque, NM, 1994.                   *
C*                                                                     *
C*  Needed Subroutines:                                                *
C*      XBEAM   Evaluates the LBRF from 3-parameter approxmation       *
C*      XBINI   Reads parameters for the LBRF approximation            *
C*      EAGRID  Creates energy and angular grid for the LBRF approx.   *
C*      SILOIO  Reads input data and outputs results for silo problem  *
C*      SILO    Calculates skyshine dose for silo geometry             *
C*      WALLIO  Reads input data and outputs results for wall problem  *
C*      WALL    Calculates skyshine dose for wall geometry             *
C*      BOXIO   Reads input data and outputs results for silo problem  *
C*      BOX     Calculates skyshine dose for silo geometry             *
C*      READGS  Gives Gaussian quadrature ordinates and weights        *
C*      GMU     Gives mass interaction coefficients                    *
C*      BUGP    Gives geometric progression buildup factors            *
C*                                                                     *
C*  Data Files Needed:  LOWGAM.DAT and HIGHGAM.DAT containing the      *
C*                      parameters for the LBRF approximation          *
C*                                                                     *
C*  Modified 7/94 from SKYBEAM code by J.K. Shultis and R.E. Faw       *
C*  Modified (1/98): corrected output table for box geometry (JKS)     *
C*  Modified (1/99): added multigroup E-spectrum capability (JKS)      *
C*  Modified (8/99): fixed minor bug in BOX geometry (JKS)             *
C***********************************************************************
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      CHARACTER*32 OUTFIL,INNAM,FD
      CHARACTER*1 ANS,dum
      CHARACTER*9 SHIELD
      CHARACTER*80 title
      COMMON/GAUS01/X(32),W(32)
      COMMON/SHLD/MAT,Z,ZMFP,SHIELD
      COMMON/ECOM/E(100),P(100),NE,title
      FD = '(1X,A,$)'
C
C****** PRINT OPENING SCREEN
      WRITE(*,20)
      WRITE(*,21)
C
C****** READ IN COEFFICIENTS FOR APPROXIMATE LBRF
      CALL XBINI
C
C****** ASK USER FOR COMMON PROBLEM DATA
C** Ask if keyboard or file input; Get output file name
      PRINT *
      WRITE(*,FD)'Enter type of data entry (=1 keyboard; =2 file) ==> '
      READ *, INTYPE
      IF(INTYPE.EQ.2) THEN
         PRINT *
         WRITE(*,FD) 'Enter name of input file for skyshine problem data
     1 ===> '
         READ(*,1) INNAM
         OPEN(18,FILE=INNAM)
         READ(18,1,END=99) OUTFIL
      ELSE
         PRINT *
         WRITE(*,FD) 'Enter output filename ==> '
         READ (*,1) OUTFIL
      ENDIF
      ISTRT=0
C
C
C*** GET INPUT DATA COMMON TO ALL GEOMETRIES
98    IF (INTYPE.EQ.1) THEN
C-- Problem Title
         WRITE(*,118)
         PRINT*, ' Enter one-line title for present case:'
         READ(*,1) title
C-- Problem geometry
         WRITE(*,16)
         READ*, IPROB
C-- Source bins and energy spectrum
         WRITE(*,117)
         READ*, NE
         WRITE(*,118)
         DO 81 I=1,NE
           WRITE(*,17) I
           READ*,E(I),P(I)
81       CONTINUE
C-- Air density
         WRITE(*,19)
         READ*,RHO
C-- No. distances
         WRITE(*,22)
         READ(6,*) NPTS
C-- Shield material
         WRITE(*,18)
         READ*, MAT
C-- Check E range: for shielded case must be less 10 MeV
         IF(MAT.GT.0 .AND. E(NE).GT.10.D0) THEN
            PRINT *, 'Energy must be <= 10 MeV for a shielded source.
     1Program stopped'
            STOP
         END IF
C-- Shield mass thickness
         IF (MAT.GT.0) THEN
            WRITE(*,23)
            READ*,Z
         ELSE
            Z=0.0D0
         ENDIF
C-- Order of integration quadrature
         PRINT *
         WRITE(*,FD) 'Enter order for Gauss quadrature (4,8,16,32) ==> '
         READ *, NGAU

C** Read data from input file
      ELSE
         READ(18,1,END=99) dum
         READ(18,1,END=99) title
         READ(18,*,END=99) IPROB,RHO,NPTS,MAT,Z,NGAU,NE
         IF(MAT.EQ.0) THEN
            Z=0.0D0
         ENDIF
         DO 82 I=1,NE
            READ(18,*,END=99)  E(I),P(I)
82       CONTINUE
      ENDIF
C
C
C*** SET CONSTANTS, SET UP QUADRATURE, OPEN OUTPUT FILE
C     RHO = 0.001200D0
      PI = DACOS(-1.D0)
      CALL READGS(NGAU,X,W)
C
      IF (ISTRT.EQ.0) THEN
         OPEN(12,FILE=OUTFIL,STATUS='UNKNOWN')
         ISTRT=1
      ENDIF
C
      NPTS = NPTS + 1
      IF(MAT.EQ.0)THEN
          SHIELD = '    none'
        ELSE IF(MAT.EQ.1) THEN
          SHIELD = '   water'
        ELSE IF(MAT.EQ.2) THEN
          SHIELD = 'concrete'
        ELSE IF(MAT.EQ.3) THEN
          SHIELD = '    iron'
        ELSE
          SHIELD = '    lead'
        END IF
C
C
C********** CALL THE APPROPRIATE GEOMETRY INPUT ROUTINE
      IF(IPROB.EQ.1) THEN
         CALL SILOIO(RHO,NGAU,NPTS,INTYPE)
      ELSEIF (IPROB.EQ.2) THEN
         CALL WALLIO(RHO,NGAU,NPTS,INTYPE)
      ELSE
         CALL BOXIO(RHO,NGAU,NPTS,INTYPE)
      ENDIF
C
C***** ASK IF ANOTHER CASE IS TO BE ANALYZED
      IF (INTYPE.EQ.1) THEN
        PRINT *
        WRITE(*,FD) 'Analyze another case? (y/n) ==> '
        READ (*,1) ANS
        IF (ANS.EQ.'Y'.OR.ANS.EQ.'y') GOTO 98
      ELSE
        GOTO 98
      ENDIF
C
C
1     FORMAT(A)
16    FORMAT (/' Enter geometry type for the source containment:',/
     1         '  (=1 source on axis of a circular silo     )',/
     2         '  (=2 source behind an infinitely long wall )',/
     3         '  (=3 source in a rectangular building      )  ==> ',$)
18    FORMAT (/' Enter material for slab shield above source:',/
     1         '  (=0 no shield    =1 water    =2 concrete  )',/
     2         '  (=3 for iron     =4 lead                  ) ==> ',$)
117   FORMAT (/' Enter number of source energy groups (<100) ==> ',$)
118   FORMAT (1X)
17    FORMAT ( ' Enter Emid (.02-100MeV) and Frequency for Group ',
     1           I2,' ==> ',$)
19    FORMAT (/' Enter air density in g/cm^3 (e.g., 0.0012) ==> ',$)
22    FORMAT (/' Enter no. of intermediate distances for calcualtions ==
     1> ',$)
23    FORMAT (/' Enter mass thickness (g/cm^2) of overhead shielding ==>
     1 ',$)
20    FORMAT(22(/),22X,23('*'),/22X,'***** SKYDOSE 2.3 *****'/
     1       22X,23('*')/)
21    FORMAT(' This program evaluates the gamma-ray skyshine dose from a
     1n isotropic,'/' monoenergetic, point, gamma-photon source  collima
     2ted by three simple'/' geometries:  an open silo,  a vertical blac
     3k wall, a rectangular'/' building. The source may be shielded by a
     4n optional overhead slab'/' shield.  The method used is based on t
     5he integral line-beam skyshine'/' method using a 3-parameter appro
     6ximation for the LBRF.'//)
C
99    STOP
      END


      SUBROUTINE SILOIO(RHO,NGAU,N,KEY)
C*****************************************************************
C  Purpose:
C   This subroutine (1) asks for input data for the open silo
C   source, (2) calls for the skyshine dose, and (3) outputs the
C   results.
C
C  Input variables:
C   RHO   =  air density (g/cm^3)
C  NGAU   =  order of quadrature
C     N   =  no. of intermeduate source-detector distances
C   KEY   =  type of geometry input (=1 keyboard; =2 file)
C
C  Silo geometry variables:
C     R   =  radius of the silo,
C     HS  =  source elevation with respect to top of the silo,
C     HD  =  detector elevation with respect to top of the silo,
C     XD  =  horizontal distance from source to detector Xd
C     Z   =  mass thickness of cover (g/cm^2) above top of the silo
C    ZMFP =  mean free paths of cover above top of the silo
C     MAT =  index for material of cover (=1 water; =2 concrete
C                                         =3 iron;  =4 lead     )
C
c*****************************************************************
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      CHARACTER*9 SHIELD,FD
      CHARACTER*80 title
      COMMON/SHLD/MAT,Z,ZMFP,SHIELD
      COMMON/ECOM/E(100),P(100),NE,title
      FD = '(/1X,A,$)'
C
C*** READ GEOMETRY DATA
      IF(KEY.EQ.2) THEN
         READ(18,*,END=41,ERR=41) HS,HD,R,XD
         IF(HS.LE.0.0D0 .OR. R.LE.0.D0 .OR. XD.LE.R) THEN
            WRITE(*,30)
            GOTO 41
         ENDIF
      ELSE
40      WRITE(*,FD)'Enter source elevation below silo top (m) Hs ==> '
        READ*, HS
        WRITE(*,FD)'Enter detector elevation below silo top (m) Hd ==> '
        READ*, HD
        WRITE(*,FD)'Enter silo inner radius (m) R ==> '
        READ*, R
        WRITE(*,FD) 'Enter horizontal source-to-detector distance (m) Xd
     * ==> '
        READ*,XD
        IF(HS.LE.0.0D0 .OR. R.LE.0.D0 .OR. XD.LE.R) THEN
          WRITE(*,30)
          PRINT*,'RE-ENTER GEOMETRY DATA FOR SILO...'
          GOTO 40
        ENDIF
      ENDIF
C
C
C** CALCULATE SOURCE COLLIMATION SOLID ANGLE
42    PI=DACOS(-1.D0)
      W0=HS/DSQRT(R*R + HS*HS)
      OMEGA=2.D0*PI*(1.D0-W0)
C
C*** PRINT GEOMETRY DATA INTO OUTPUT FILE
      WRITE(12,15) title,RHO,HS,HD,R,OMEGA,XD,SHIELD,Z,NGAU
      WRITE(6,15) title,RHO,HS,HD,R,OMEGA,XD,SHIELD,Z,NGAU
      WRITE(12,16) (E(K),P(K),K=1,NE)
      WRITE(6,16) (E(K),P(K),K=1,NE)
      WRITE(12,25)
      WRITE(6,25)
C
C********* BEGIN CALCULATIONS FOR ALL DISTANCES
      DO 888 I=1,N
         X = (XD/REAL(N))*I
         XI=DATAN((HS-HD)/X)
         DD=DSQRT((HS-HD)**2+X*X)
C        --- Check if detector is inside the silo
         IF (X.LT.R) THEN
            WRITE(12,31)
            WRITE(*,31)
         ENDIF
C        --- Check if source can be seen by detector
         IF ( (HS/R).LE.(HS-HD)/X ) THEN
            WRITE(12,32)
            WRITE(*,32)
         ENDIF
C
C        --  Sum over each energy bin
         SUME=0.0D0
         DO 10, J=1,NE
            ENRGY = E(J)
            ZMFP = Z*GMU(ENRGY,1,MAT+1)
            CALL SILO(ENRGY,RHO,NGAU,W0,DD,XI,DOSE)
            SUME = SUME + DOSE*P(J)
10       CONTINUE
         DOSE = SUME
C
C*** PRINT OUT RESULTS
         DOSE2 = 1.145*DOSE
         DOSE3=DD*DD*DOSE*1.145/OMEGA
C        DEN = 100*RHO
         WRITE(12,20)DD,100.D0*DD*RHO,DOSE,DOSE2,DOSE3
         WRITE(6,20) DD,100.D0*DD*RHO,DOSE,DOSE2,DOSE3
 888  CONTINUE
      WRITE(12,26)
      WRITE(6,26)
      RETURN
C
C** Error in input data file
41    PRINT*,'**** Error in data file: Program stopped'
      STOP
C
C
15    FORMAT(///A80,/'****** SKYDOSE 2.3: Silo Geometry '/
     2       '  Air density (g/cm^3)       =',F10.6,/
     3       '  Source elevation (m)       =',F10.2,/
     4       '  Detector elevation (m)     =',F10.2,/
     5       '  Silo radius (m)            =',F10.2,/
     6       '  Source collimation (sr)    =',F10.4,/
     7       '  Source-detector dist. (m)  =',F10.1,/
     7       '  Overhead shield material   =  ',A8,/
     7       '  Shield thickness (g/cm^2)  =',F10.2,/
     8       '  Gauss quadrature order     =',I10,/
     9       '  3-parm. approx. LBRF used ',/)
16    FORMAT(' Energy Spectrum: Emid (MeV) and Frequency:'
     1       /(15X,F12.4,G15.4))
20    FORMAT(F12.0,F12.2,2X,3(1PE13.4))
25    FORMAT(/27X,'SKYSHINE DOSES',/2X,66('-'),/
     1  6X,'S-D(m)',6X,'g/cm^2',5X,'rad/photon',4X,'R/photon',5X,
     2  'R m^2/sr',/2x,66('-'))
26    FORMAT(2X,66('-'))
30    FORMAT(/' **** Error in geometry input data:',/
     1        '      It is required that  (1) Hs > 0',/
     2        '                           (2) R  > 0',/
     3        '                           (3) Xd > R')
31    FORMAT(' **** Warning: Detector inside silo at this distance')
32    FORMAT(' **** Warning: Detector sees source at this distance')
      END


       SUBROUTINE SILO(E,RHO,NGAU,W0,ZZ,XI,DOSE)
C********************************************************************
C  Purpose:
C   This subroutine calculates the skyshine dose for a source in
C   an open cylindrical silo on the silo's axis.
C
C  Input:
C     E = source photon energy (MeV)
C   RHO =  air density (g/cm^3)
C  NGAU = Gaussian quadrature order
C    W0 = cosine of the maximun polar angle (silo collimation)
C    ZZ = source-to-detector distance (m)
C    XI = angle between the horizontal axis and source-detector axis
C
C  Output:
C    DOSE = skyshine dose (rad/photon)
C
C   NEEDED SUBROUTINES:
C      BUGP  = Buildup factor (geometric progression)
C      GMU   = Mass interaction coefficients
C      XBEAM = Approximate LBRF
C     or
C       BEAM = LBRF obtained by point-kernel method,
C********************************************************************
       IMPLICIT DOUBLE PRECISION (A-H,O-Z)
       CHARACTER DUMMY*9
       COMMON/GAUS01/X(32),W(32)
       COMMON/SHLD/MAT,Z,ZMFP,DUMMY
C
      PI = DACOS(-1.D0)
C*** INTEGRATE LBRF OVER ALL AZIMUTHAL ANGLES 0 TO PI
      TOTSUM = 0.0D0
      DO 100 L = 1,NGAU
         WPSI = 0.5*PI*W(L)
         PSI = 0.5*PI*(X(L) + 1.D0)
C
C*** INTEGRATE OVER POLAR ANGLES
         TOTANS=0.0D0
         DO 99 K = 1,NGAU
           WTHE = 0.5*(1.D0 - W0)*W(K)
           WW = 0.5*(1.D0 - W0)*X(K) + 0.5*(1.D0 + W0)
           IF(WW.GT.1.D0)GOTO 110
           DWW = (1.D0 - WW*WW)**0.5
C*** CALCULATE BEAM EMISSION ANGLE PHI
           DPHI = DCOS(PSI)*DWW*DCOS(XI)+WW*DSIN(XI)
           PHI = DACOS(DPHI)*180./PI
C*** CALCULATE SKYSHINE DOSE FOR GIVEN BEAM
           CALL XBEAM(E,PHI,ZZ,RHO,RESP)
           IF(MAT.GT.0) THEN
              TSLNT = ZMFP/WW
              RESP=RESP*DEXP(-TSLNT)*BUGP(MAT+1,E,TSLNT,IERR)
           ENDIF
           TOTANS = TOTANS + WTHE*RESP
99      CONTINUE
C
        TOTSUM = TOTSUM + WPSI*TOTANS
100   CONTINUE
C
      DOSE=0.5*TOTSUM/PI
110   RETURN
      END


      SUBROUTINE WALLIO(RHO,NGAU,N,KEY)
C******************************************************************
C  Purpose:
C   This subroutine (1) asks for input data for a point isotropic
C   monoenergetic source behind an infintely long vertical wall,
C   (2) calls for the skyshine dose, and (3) outputs the results.
C
C  Input variables:
C   RHO   =  air density (g/cm^3)
C  NGAU   =  order of quadrature
C     N   =  no. of intermeduate source-detector distances
C   KEY   =  type of geometry input (=1 keyboard; =2 file)
C
C  Other subroutine variables:
C     R   =  perped cular source-to-wall distance (m)
C     HS  =  source elevation below top of wall(m),
C     HD  =  detector elevation below top of wall(m),
C     ZD  =  source-detector offset (parallel to top of wall) (m),
C     X   =  horizontal source-to-detector distance (m)
C    ZMFP =  mean free paths of cover above top of the silo
C     MAT =  index for material of cover (=1 water; =2 concrete
C                                         =3 iron;  =4 lead     )
C
c******************************************************************
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      CHARACTER*9 SHIELD,FD
      CHARACTER*80 title
      COMMON/SHLD/MAT,ZSHLD,ZMFP,SHIELD
      COMMON/ECOM/E(100),P(100),NE,title
      FD = '(/1X,A,$)'
C
C*** READ GEOMETRY DATA
      IF(KEY.EQ.2) THEN
         READ(18,*,END=41,ERR=41) HS,HD,R,ZD,XD
         IF(HS.LE.0.0D0 .OR. R.LE.0.D0 .OR. XD.LE.R) THEN
            WRITE(*,30)
            GOTO 41
         ENDIF
      ELSE
40     WRITE(*,FD)'Enter source elevation below top of wall (m) Hs ==> '
       READ*,HS
       WRITE(*,FD) 'Enter detector elevation below top of wall (m) Hd ==
     1> '
       READ*,HD
       WRITE(*,FD) 'Enter perpendicular distance of source from wall (m)
     1 R ==> '
       READ*,R
       WRITE(*,FD) 'Enter horizontal detector offset from source-wall no
     1rmal (m) Zd ==> '
       READ*,ZD
       WRITE(*,FD)'Enter horiz. source-detector dist. along source-wall
     1normal (m) Xd ==> '
       READ*,XD
       IF(HS.LE.0.0D0 .OR. R.LE.0.D0 .OR. XD.LE.R) THEN
         WRITE(*,30)
         PRINT*,'RE-ENTER GEOMETRY DATA FOR WALL GEOMETRY...'
         GOTO 40
       ENDIF
      ENDIF
      PI=DACOS(-1.D0)
C
C*** PRINT GEOMETRY DATA INTO OUTPUT FILE
      WRITE(12,15) title,RHO,HS,HD,R,ZD,XD,SHIELD,ZSHLD,NGAU
      WRITE(6,15) title,RHO,HS,HD,R,ZD,XD,SHIELD,ZSHLD,NGAU
      WRITE(12,16) (E(K),P(K),K=1,NE)
      WRITE(6,16) (E(K),P(K),K=1,NE)
      WRITE(12,25)
      WRITE(6,25)
C
C********* BEGIN CALCULATIONS FOR ALL DISTANCES
       DO 888 I=1,N
         X = (XD / REAL(N))*I
         ZETA=DATAN(ZD/X)
         XI=DATAN((HS-HD)/DSQRT(X*X + ZD*ZD))
         DD=DSQRT((HS-HD)**2 + X*X + ZD*ZD)
C        --- Check if detector is on source side of wall
         IF (X.LT.R) THEN
            WRITE(12,31)
            WRITE(*,31)
         ENDIF
C        --- Check if source can be seen by detector
         IF ( (HS/R).LE.(HS-HD)/X ) THEN
            WRITE(12,32)
            WRITE(*,32)
         ENDIF
C
C        --  Sum over each energy bin
         SUME=0.0D0
         DO 10, J=1,NE
            ENRGY = E(J)
            ZMFP = ZSHLD*GMU(ENRGY,1,MAT+1)
            CALL WALL(ENRGY,RHO,NGAU,R,HS,DD,XI,ZETA,DOSE)
            SUME = SUME + DOSE*P(J)
10       CONTINUE
         DOSE = SUME
C
         DOSE2 = DOSE*1.145
         WRITE(12,20) X,DD,DOSE,DOSE2
         WRITE(6,20)  X,DD,DOSE,DOSE2
888   CONTINUE
      WRITE(12,26)
      WRITE(6,26)
      RETURN
C
C** Error in input data file
41    PRINT*,'**** Error in data file: Program stopped'
      STOP
C
C
15    FORMAT(///A80,/'****** SKYDOSE 2.3: Wall Geometry'/
     2       '  Air density (g/cm^3)       =',F10.6,/
     3       '  Source elevation (m)       =',F10.2,/
     4       '  Detector elevation (m)     =',F10.2,/
     5       '  Source-to-wall distance(m) =',F10.2,/
     6       '  Detector horiz. offset (m) =',F10.2,/
     7       '  Source-detector dist. (m)  =',F10.1,/
     7       '  Overhead shield material   =  ',A8,/
     7       '  Shield thickness (g/cm^2)  =',F10.2,/
     8       '  Gauss quadrature order     =',I10,/
     9       '  3-parm. approx. LBRF used ',/)
16    FORMAT(' Energy Spectrum: Emid (MeV) and Frequency:'
     1       /(15X,F12.4,G15.4))
20    FORMAT(2F12.1,2X,2(1PE13.4))
25    FORMAT(/21X,'SKYSHINE DOSES',/2X,53('-'),/
     1  8X,'X(m)',6X,'S-D(m)',5X,'rad/photon',4X,'R/photon',/2X,53('-'))
26    FORMAT(2X,53('-'))
30    FORMAT(/' **** Error in geometry input data:',/
     1        '      It is required that  (1) Hs > 0',/
     2        '                           (2) R  > 0',/
     3        '                           (3) Xd > R')
31    FORMAT(' **** Warning: Detector on source side at this distance')
32    FORMAT(' **** Warning: Detector sees source at this distance')
      END


      SUBROUTINE WALL(E,RHO,NGAU,R,HS,ZZ,XI,ZETA,DOSE)
C*******************************************************************
C Purpose:
C   This subroutine calculates the skyshine dose for the infinite
C   wall problem by integrating the line-beam response function
C   over all directions allowed by the source-wall geometry.
C   All integration are performed by Gaussian quadrature.
C
C  Input:
C     E = source photon energy (MeV)
C   RHO =  air density (g/cm^3)
C  NGAU = Gaussian quadrature order for the integration
C     R = perpedicular source-to-wall distance (m)
C    HS = source elevation belwow top of wall (m)
C    ZZ = source-to-detector distance along normal from source (m)
C    XI = angle between the horizontal and the source-detector axis
C  ZETA = offset angle (between the normal axis and horizontal axis)
C
C  Output:
C    DOSE = Skyshine Dose (rad/photon)
C
C*  NEEDED SUBROUTINES:
C      BUGP  = Buildup factor (geometric progression)
C      GMU   = Mass interaction coefficients
C*     XBEAM = Approximate LBRF
C     or:
C*      BEAM = LBRF obtained by point-kernel method,
C********************************************************************
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      CHARACTER DUMMY*9
      COMMON/GAUS01/X(32),W(32)
      COMMON/SHLD/MAT,Z,ZMFP,DUMMY
C
      FPSI(U,V,Y)=DCOS(U)*DSQRT(1.D0-V*V)*DCOS(Y)+V*DSIN(Y)
      PI=DACOS(-1.D0)
C
C**** For the range 0< PSI < PI/2+ZETA
      SUM12 = 0.0D0
      DO 20 I = 1,NGAU
         B = PI/2.D0 + ZETA
         PSI = 0.5D0*B*(X(I) + 1.D0)
         W1 = 0.5D0*B*W(I)
         SUM11 = 0.0D0
         DO 10 J = 1,NGAU
            A = DATAN( R/(HS*DCOS(PSI - XI)))
            W0 = DCOS(A)
            WW = 0.5D0*(1. - W0)*X(J) + 0.5D0*(1.D0 + W0)
            W2 = 0.5*(1. - W0)*W(J)
            DPHI = FPSI(PSI,WW,XI)
            PHI = DACOS(DPHI)*180.D0/PI
            CALL XBEAM(E,PHI,ZZ,RHO,RESP)
            IF(MAT.GT.0) THEN
               TSLNT = ZMFP/WW
               RESP=RESP*DEXP(-TSLNT)*BUGP(MAT+1,E,TSLNT,IERR)
            ENDIF
            SUM11 = SUM11 + W2*RESP
10       CONTINUE
         SUM12 = SUM12 + W1*SUM11
20    CONTINUE
C
C**** For the range PI/2 + ZETA < PSI < 3PI/2 + ZETA
      SUM22 = 0.0D0
      DO 40 I = 1,NGAU
         B = 3*PI/2.D0 + ZETA
         A = PI/2.D0 + ZETA
         PSI = 0.5D0*(B-A)*X(I) + 0.5D0*(B+A)
         W1 = 0.5D0*(B-A)*W(I)
         SUM21 = 0.0D0
         DO 30 J=1,NGAU
            WW = 0.5D0*(X(I)+1.D0)
            DPHI = FPSI(PSI,WW,XI)
            PHI = DACOS(DPHI)*180.D0/PI
            W2 = 0.5D0*W(J)
            CALL XBEAM(E,PHI,ZZ,RHO,RESP)
            IF(MAT.GT.0) THEN
               TSLNT = ZMFP/WW
               RESP=RESP*DEXP(-TSLNT)*BUGP(MAT+1,E,TSLNT,IERR)
            ENDIF
            SUM21 = SUM21 + W2*RESP
30       CONTINUE
         SUM22 = SUM22 + W1*SUM21
40    CONTINUE
C
C*** For the range 3 PI/2 + ZETA < PSI < 2 PI
      SUM32 = 0.0D0
      DO 60 I = 1,NGAU
         B = 2.D0*PI
         A = 1.5D0*PI + ZETA
         PSI = 0.5D0*(B-A)*X(I)+0.5D0*(B+A)
         W1 = 0.5D0*(B-A)*W(I)
         SUM31 = 0.0D0
         DO 50 J = 1,NGAU
            A = DATAN( R/(HS*DCOS(PSI-XI)))
            W0 = DCOS(A)
            WW = 0.5*(1.D0-W0)*X(J) + 0.5*(1.+W0)
            W2 = 0.5*(1.D0-W0)*W(J)
            DPHI = FPSI(PSI,WW,XI)
            PHI = DACOS(DPHI)*180./PI
            CALL XBEAM(E,PHI,ZZ,RHO,RESP)
            IF(MAT.GT.0) THEN
               TSLNT = ZMFP/WW
               RESP=RESP*DEXP(-TSLNT)*BUGP(MAT+1,E,TSLNT,IERR)
            ENDIF
            SUM31 = SUM31 + W2*RESP
50       CONTINUE
         SUM32 = SUM32 + W1*SUM31
60    CONTINUE
C
C**** ADD THREE COMPONENTS TO GET TOTAL SKYSHINE DOSE
      DOSE = SUM12 + SUM22 + SUM32
      DOSE = DOSE/4.D0/PI
      RETURN
      END
C
C
      SUBROUTINE BOXIO(RHO,NGAU,N,KEY)
C******************************************************************
C
C  Purpose:
C   This subroutine (1) asks for input data for the source in a
C   rectangular building problem, (2) calls for the skyshine dose,
C   and (3) outputs the results.
C
C  Input variables:
C   RHO   =  air density (g/cm^3)
C  NGAU   =  order of quadrature
C     N   =  no. of intermeduate source-detector distances
C   KEY   =  type of geometry input (=1 keyboard; =2 file)
C
C  Box geometry variables:
C     HS  =  source elevation below top of building (m)
C     HD  =  detector elevation below top of building (m)
C     X1  =  perpendicular distance from source to rear wall (m)
C     X2  =  perpendicular distance from source to front wall (m)
C     Y1  =  perpendicular distance from source to left wall (m)
C     Y2  =  perpendicular distance from source to right wall (m)
C     Z   =  source-detector offset (parallel to top of wall)(m)
C     X   =  horizontal source-to-detector distance (m)
C    ZMFP =  mean free paths of cover above top of the silo
C     MAT =  index for material of cover (=1 water; =2 concrete
C
c******************************************************************
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION PSI(5)
      CHARACTER*9 SHIELD,FD
      CHARACTER*80 title
      COMMON/SHLD/MAT,Z,ZMFP,SHIELD
      COMMON/ECOM/E(100),P(100),NE,title
      FD = '(/1X,A,$)'
C
C*** READ GEOMETRY DATA
      IF(KEY.EQ.2) THEN
         READ(18,*,END=41,ERR=41) HS,HD,X1,X2,Y1,Y2,YD,XD
         IF(HS.LE.0.0D0.OR.XD.LE.0.D0.OR.X1.LE.0.D0.OR.X2.LE.0.D0.OR.
     1     Y1.LE.0.D0 .OR. Y2.LE.0.D0 .OR. YD.LT.-Y2 .OR. YD.GT.Y1)THEN
            WRITE(*,30)
            GOTO 41
         ENDIF
      ELSE
40      WRITE(*,FD)'Enter source elevation below top of building (m) Hs
     *==> '
        READ*,HS
        WRITE(*,FD)'Enter detector elevation below building top (m) Hd =
     *=> '
        READ*,HD
        WRITE(*,FD)'Enter perp. distance from source to rear wall (m) X1
     * ==> '
        READ*,X1
        WRITE(*,FD)'Enter perp. distance from source to front wall (m) X
     *2 ==> '
        READ*,X2
        WRITE(*,FD)'Enter perp. distance from source to left wall (m) Y1
     * ==> '
        READ*,Y1
        WRITE(*,FD)'Enter perp. distance from source to right wall (m) Y
     *2 ==> '
        READ*,Y2
        WRITE(*,FD)'Enter detector offset from normal to front wall (m)
     *Yd ==> '
        READ*,YD
        WRITE(*,FD)'Enter source-detector dist. along normal to front wa
     *ll (m) Xd ==> '
        READ*,XD

        IF(HS.LE.0.0D0.OR.XD.LE.0.D0.OR.X1.LE.0.D0.OR.X2.LE.0.D0.OR.
     1    Y1.LE.0.D0 .OR. Y2.LE.0.D0 .OR. YD.LT.-Y2 .OR. YD.GT.Y1)THEN
          WRITE(*,30)
          PRINT*,'RE-ENTER GEOMETRY DATA FOR RECTANGULAR BUILDING...'
          GOTO 40
        ENDIF
      ENDIF
      PI = DACOS(-1.D0)
C
C*** PRINT GEOMETRY DATA INTO OUTPUT FILE
      WRITE(12,15) title,RHO,HS,HD,X1,X2,Y1,Y2,YD,XD,SHIELD,Z,NGAU
      WRITE(6,15)  title,RHO,HS,HD,X1,X2,Y1,Y2,YD,XD,SHIELD,Z,NGAU
      WRITE(12,16) (E(K),P(K),K=1,NE)
      WRITE(6,16) (E(K),P(K),K=1,NE)
      WRITE(12,25)
      WRITE(6,25)
C
C********* BEGIN CALCULATIONS FOR ALL DISTANCES
      DO 888 I=1,N
         X = (XD / REAL(N))*I +X1
         ZETA=DATAN(YD/X)
         PSI(1) = DATAN(Y1/X2)-ZETA
         PSI(2) = 0.5*PI+DATAN(X1/Y1)-ZETA
         PSI(3) = PI+DATAN(Y2/X1)-ZETA
         PSI(4) = 2.D0*PI-DATAN(Y2/X2)-ZETA
         PSI(5) = -DATAN(Y2/X2)-ZETA
         XI=DATAN((HS-HD)/DSQRT(X*X + YD*YD))
         DD=DSQRT((HS-HD)**2 + X*X + YD*YD)
C        --- Check if detector is inside building
         IF (X.LT.X2) THEN
            WRITE(12,31)
            WRITE(*,31)
         ENDIF
C        --- Check if source can be seen
         IF ( (HS/X2).LE.(HS-HD)/(X2+XD) ) THEN
            WRITE(12,32)
            WRITE(*,32)
         ENDIF
C
C        --  Sum over each energy bin
         SUME=0.0D0
         DO 10, J=1,NE
            ENRGY = E(J)
            ZMFP = Z*GMU(ENRGY,1,MAT+1)
            CALL BOX(ENRGY,RHO,NGAU,X1,X2,Y1,Y2,HS,DD,XI,PSI,ZETA,DOSE)
            SUME = SUME + DOSE*P(J)
10       CONTINUE
         DOSE = SUME
C
         DOSE2 = DOSE*1.145
        WRITE(12,20) X,DD,DOSE,DOSE2
        WRITE(6,20)  X,DD,DOSE,DOSE2
888   CONTINUE
      WRITE(12,26)
      WRITE(6,26)
      RETURN
C
C** Error in input data file
41    PRINT*,'**** Error in data file: Program stopped'
      STOP
C
C
15    FORMAT(///A80,/'****** SKYDOSE 2.3: Rectangular Building Geometry'
     2 ,/    '  Air density (g/cm^3)            =',F10.6,/
     3       '  Source elevation (m)            =',F10.2,/
     3       '  Detector elevation (m)          =',F10.2,/
     4       '  Source-rear wall distance (m)   =',F10.2,/
     4       '  Source-front wall distance (m)  =',F10.2,/
     4       '  Source-left wall distance (m)   =',F10.2,/
     4       '  Source-right wall distance (m)  =',F10.2,/
     5       '  Detector offset from normal (m) =',F10.2,/
     6       '  Detector-front wall distance(m) =',F10.2,/
     7       '  Overhead shield material        =  ',A8,/
     7       '  Shield mass thickness (g/cm^2)  =',F10.2,/
     8       '  Gauss quadrature order          =',I10,/
     9       '  3-parm. approx. LBRF used ',/)
16    FORMAT(' Energy Spectrum: Emid (MeV) and Frequency:'
     1       /(15X,F12.4,G15.4))
20    FORMAT(2F12.1,2X,2(1PE13.4))
25    FORMAT(/21X,'SKYSHINE DOSES',/2X,53('-'),/
     1  8X,'X(m)',6X,'S-D(m)',5X,'rad/photon',4X,'R/photon',/2X,53('-'))
26    FORMAT(2X,53('-'))
30    FORMAT(/' **** Error in geometry input data:',/
     1        '      It is required that  (1) Hs > 0',/
     2        '                           (2) Xd > 0',/
     3        '                           (3) X1,X2 > 0' ,/
     4        '                           (4) Y1,Y2 > 0' ,/
     5        '                           (5) -Y2 < Yd < Y1')
31    FORMAT(' **** Warning: Detector inside building at this distance')
32    FORMAT(' **** Warning: Detector sees source at this distance')
       END


      SUBROUTINE BOX(E,RHO,NGAU,X1,X2,Y1,Y2,HS,ZZ,XI,PSI,ZETA,DOSE)
C****************************************************************
C Purpose:
C   This subroutine calculates the skyshine dose from a point
C   isotropic monoenergetic source located in a rectangular
C   building.  The dose is calculated by numerically integrating
C   the line beam response function over all allowed emission
C   directions.
C
C  Input:
C      E  =  source photon energy (MeV)
C   NGAU  =  Gaussian quadrature order
C     X1  =  perpendicular distance from source to rear wall (m)
C     X2  =  perpendicular distance from source to front wall (m)
C     Y1  =  perpendicular distance from source to left wall (m)
C     Y2  =  perpendicular distance from source to right wall (m)
C     HS  =  source elevation with respect to top of wall (m)
C     ZZ  =  Source-to-detector distance (m)
C     XI  =  Angle between horiz. and source-detector axis (deg.)
C    PSI  =  Azimuthal angles defined in source plane (deg.)
C    ZETA =  Offset angle (between normal axis and horiz. axis)(deg)
C
C  Output:
C    DOSE = Skyshine Dose
C
C   NEEDED SUBROUTINES:
C      BUGP  = Buildup factor (geometric progression)
C      GMU   = Mass interaction coefficients
C      XBEAM = Approximate LBRF
C     or:
C       BEAM = LBRF obtained by point-kernel method
C****************************************************************
C
       IMPLICIT DOUBLE PRECISION (A-H,O-Z)
       CHARACTER DUMMY*9
       COMMON/GAUS01/X(32),W(32)
       DIMENSION PSI(5)
       COMMON/SHLD/MAT,Z,ZMFP,DUMMY

      FPSI(U,V,Y)=DCOS(U)*DSQRT(1.D0-V*V)*DCOS(Y)+V*DSIN(Y)
      PI = DACOS(-1.D0)
C
C**** for the range PSI(5) < PSI < PSI(1)
      SUM12 = 0.0D0
      DO 20 I = 1,NGAU
        A = PSI(5)
        B = PSI(1)
        PPSI = 0.5D0*((B-A)*X(I) + (B+A))
        W1 = 0.5D0*(B-A)*W(I)
        SUM11 = 0.D0
        DO 10 J = 1,NGAU
           THETA1 = DATAN(X2/(HS*DCOS(PPSI + ZETA)))
           W0 = DCOS(THETA1)
           WW = 0.5D0*((1.D0-W0)*X(J) + (1.D0+W0))
           W2 = 0.5D0*(1.D0-W0)*W(J)
           DPHI = FPSI(PPSI,WW,XI )
           PPHI = DACOS(DPHI)*180.D0/PI
           CALL XBEAM(E,PPHI,ZZ,RHO,RESP)
           IF(MAT.GT.0) THEN
              TSLNT = ZMFP/WW
              RESP=RESP*DEXP(-TSLNT)*BUGP(MAT+1,E,TSLNT,IERR)
           ENDIF
           SUM11 = SUM11 + W2*RESP
 10     CONTINUE
        SUM12 = SUM12 + W1*SUM11
20    CONTINUE
C
C**** For the range PSI(2) < PSI < PSI(1)
      SUM22 = 0.D0
      DO 40 I = 1,NGAU
        A = PSI(1)
        B = PSI(2)
        PPSI = 0.5D0*((B-A)*X(I) + (B+A))
        W1 = 0.5D0*(B-A)*W(I)
        SUM21 = 0.D0
        DO 30 J = 1,NGAU
           THETA2 = DATAN(Y1/(HS*DSIN(PPSI+ZETA)))
           W0 = DCOS(THETA2)
           WW = 0.5D0*((1.D0-W0)*X(J) + (1.D0+W0))
           W2 = 0.5*(1.D0 - W0)*W(J)
           DPHI = FPSI(PPSI,WW,XI)
           PPHI = DACOS(DPHI)*180.D0/PI
           CALL XBEAM(E,PPHI,ZZ,RHO,RESP)
           IF(MAT.GT.0) THEN
              TSLNT = ZMFP/WW
              RESP=RESP*DEXP(-TSLNT)*BUGP(MAT+1,E,TSLNT,IERR)
           ENDIF
           SUM21 = SUM21 + W2*RESP
 30     CONTINUE
        SUM22 = SUM22 + W1*SUM21
 40   CONTINUE
C
C**** For the range PSI(3) < PSI < PSI(2)
      SUM32 = 0.D0
      DO 60 I = 1,NGAU
        A = PSI(2)
        B = PSI(3)
        PPSI = 0.5D0*((B-A)*X(I) + (B+A))
        W1 = 0.5D0*(B-A)*W(I)
        SUM31 = 0.D0
        DO 50 J = 1,NGAU
           THETA3 = DATAN(X1/(-HS*DCOS(PPSI+ZETA)))
           W0 = DCOS(THETA3)
           WW = 0.5D0*((1.D0-W0)*X(J) + (1.D0+W0))
           W2 = 0.5D0*(1.D0-W0)*W(J)
           DPHI = FPSI(PPSI,WW,XI)
           PPHI = DACOS(DPHI)*180.D0/PI
           CALL XBEAM(E,PPHI,ZZ,RHO,RESP)
           IF(MAT.GT.0) THEN
              TSLNT = ZMFP/WW
              RESP=RESP*DEXP(-TSLNT)*BUGP(MAT+1,E,TSLNT,IERR)
           ENDIF
           SUM31 = SUM31 + W2*RESP
 50     CONTINUE
        SUM32 = SUM32 + W1*SUM31
 60   CONTINUE
C
C**** For the range PSI(4) < PSI < PSI(3)
      SUM42 = 0.D0
      DO 80 I = 1,NGAU
        A = PSI(3)
        B = PSI(4)
        PPSI = 0.5D0*((B-A)*X(I) + (B+A))
        W1 = 0.5D0*(B-A)*W(I)
        SUM41 = 0.D0
        DO 70 J = 1,NGAU
           THETA4 = DATAN(Y2/(-HS*DSIN(PPSI+ZETA)))
           W0 = DCOS(THETA4)
           WW = 0.5D0*((1.D0-W0)*X(J) + (1.D0+W0))
           W2 = 0.5D0*(1.D0-W0)*W(J)
           DPHI = FPSI(PPSI,WW, XI)
           PPHI = DACOS(DPHI)*180.D0/PI
           CALL XBEAM(E,PPHI,ZZ,RHO,RESP)
           IF(MAT.GT.0) THEN
              TSLNT = ZMFP/WW
              RESP=RESP*DEXP(-TSLNT)*BUGP(MAT+1,E,TSLNT,IERR)
           ENDIF
           SUM41 = SUM41 + W2*RESP
 70     CONTINUE
        SUM42 = SUM42 + W1*SUM41
 80   CONTINUE
C
C**** Sum four components to get total skyshine dose
      DOSE = SUM12 + SUM22 + SUM32 + SUM42
      DOSE = 0.25D0*DOSE/PI
      RETURN
      END



      BLOCK DATA EAGRID
C*************************************************************************
C  Initialize data for XBEAM
C*************************************************************************
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON /GAMVEC/ EEX(2,15),PPSIX(2,20)
C***** Discrete angles for calculation of approximate lbrf
C      Energy < 10.0 MeV
        DATA (PPSIX(1,I),I=1,20)
     *       /0.5,1.5,2.5,4.0,6.0,8.5,12.5,17.5,25.0,35.0,45.0,
     *        55.0,65.0,75.0,85.0,95.0,110.,130.,150.,170./
C      Energy > 10.0 MeV
        DATA (PPSIX(2,I),I=1,17)
     *       /2.0,3.5,5.0,7.5,10.0,15.0,20.0,30.0,45.0,60.0,
     *        75.0,85.0,95.0,105.0,120.0,135.0,150.0/
C***** Discrete energies for calculation of approximate lbrf
C      Energy < 10.0 MeV
        DATA (EEX(1,I),I=1,15)
     *       /0.02,0.03,0.04,0.06,0.08,0.1,0.2,0.4,0.7,1.0,
     *        2.0,4.0,7.0,10.,15./
C      Energy > 10.0 MeV
        DATA (EEX(2,I),I=1,5)/10.0,20.0,40.0,70.0,100.0/
       END


      SUBROUTINE XBINI
C**********************************************************************
C Reads data files containing coefficients for the three-paramter
C approximation to the LBRF.  Call once before using subprogram XBEAM.
C**********************************************************************
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      CHARACTER*15 EHDR
      COMMON /GAMDAT/ G(2,15,20,3),ANGL(2,20)
C
C ****** Read in the fit parameters for the approximate LBRF
      OPEN(21,FILE='LOWGAM.DAT')
      OPEN(23,FILE='HIGHGAM.DAT')
C** Read low-E 3-parm data
      DO 20 I=1,15
        READ(21,10) EHDR
        READ(21,*)(ANGL(1,J),G(1,I,J,1),G(1,I,J,2),G(1,I,J,3),J=1,20)
20    CONTINUE
C** Read high-E 3-parm data
      DO 30 I=1,5
        READ(23,10) EHDR
        READ(23,*)(ANGL(2,J),G(2,I,J,1),G(2,I,J,2),G(2,I,J,3),J=1,17)
30    CONTINUE

10    FORMAT(A)
      CLOSE(21)
      CLOSE(23)
      RETURN
      END



      SUBROUTINE XBEAM(E,PSI,ZZ,RHOACT,RESP)
C***************************************************************************
C  Purpose:
C    This subroutine calculates the skyshine dose at a detector
C    arising from a monoenergetic and monodirectional photon beam
c    using a 3-parameter approximation for the LBRF.
C
C  Input:
C      E = Source photon energy (MeV),
C    PSI = photon emission angle (deg.)
C     ZZ = source-to-detector distance (m)
C RHOACT = air density (g cm^-3)
C
C  Output:
C     RESP = Line-beam response function (rad/photon)
C
C  Variables:
C        FK     = flux-to-dose conversion factor
C        IE     = energy group index in which energy E resides
C        IEADJ  = adjacent enrgy index
C        JP     = angular group index in which a beam angle reside
C        JPADJ  = adjacent angle index
C
C  Needed Routines:  XBINI to read LBRF approximation coefficients
C                    EAGRID Block Data to set up E and ANG grid
C
C**************************************************************************

        IMPLICIT DOUBLE PRECISION (A-H,O-Z)

        DIMENSION  FF(20),FIE(20),FIEADJ(20)
        COMMON /GAMDAT/ G(2,15,20,3),ANGL(2,20)
        COMMON /GAMVEC/ EE(2,15),PPSI(2,20)

        IF(E.LE.15.0) THEN
          IDAT = 1
          NE = 15
          NANG = 20
        ELSE
          IDAT = 2
          NE = 5
          NANG = 17
        END IF

        FK=1.308E-11
        RHOAIR = 0.0012D0
        RFACT = RHOACT/RHOAIR

C** Find energy indices IE and IEADJ bracketing energy E

        IF (E.LE.EE(IDAT,1)) THEN
            IE = 1
        ELSEIF (E.GE.EE(IDAT,NE)) THEN
            IE = NE
        ELSE
            DO 10 I=2,NE
              IF(E.GE.EE(IDAT,I-1).AND.E.LT.EE(IDAT,I)) THEN
                 IE = I-1
                 GOTO 11
              ENDIF
  10        CONTINUE
        ENDIF
  11    EIE = EE(IDAT,IE)
        IEADJ = IE+1
        IF(IE.EQ.NE) IEADJ = NE-1

C*** Find angular indices JP and JPADJ bracketing angle PSI

        IF (PSI.LE.PPSI(IDAT,1)) THEN
            JP = 1
        ELSEIF (PSI.GE.PPSI(IDAT,NANG)) THEN
            JP = NANG
        ELSE
            DO 20 I=2,NANG
              IF(PSI.GE.PPSI(IDAT,I-1).AND.PSI.LT.PPSI(IDAT,I)) THEN
                 JP = I-1
                 GOTO 21
              ENDIF
  20        CONTINUE
        ENDIF
  21    JPADJ = JP + 1
        IF (JP.EQ.NANG) JPADJ = NANG-1

C** Evaluate LBRF at four surrounding tabulated grid points
        X=ZZ*RFACT
        FIE(JP)=DEXP(G(IDAT,IE,JP,1) - G(IDAT,IE,JP,3)*X)
     *          *X**G(IDAT,IE,JP,2)
        FIEADJ(JP)=DEXP(G(IDAT,IEADJ,JP,1)-G(IDAT,IEADJ,JP,3)*X)
     *             *X**G(IDAT,IEADJ,JP,2)
        FIE(JPADJ)=DEXP(G(IDAT,IE,JPADJ,1)-G(IDAT,IE,JPADJ,3)*X)
     *             *X**G(IDAT,IE,JPADJ,2)
        FIEADJ(JPADJ)=DEXP(G(IDAT,IEADJ,JPADJ,1)
     *                                - G(IDAT,IEADJ,JPADJ,3)*X)
     *             *X**G(IDAT,IEADJ,JPADJ,2)
C
C** Interpolate between bracketing energies
        EFACT=(E-EE(IDAT,IE))/(EE(IDAT,IEADJ)-EE(IDAT,IE))
        FF(JP)=FIE(JP)+(FIEADJ(JP)-FIE(JP))*EFACT
        FF(JPADJ)=FIE(JPADJ)+(FIEADJ(JPADJ)-FIE(JPADJ))*EFACT

C** Log-interpolate between the bracketing angles
        FF(JP)=DLOG(FF(JP))
        FF(JPADJ)=DLOG(FF(JPADJ))
        RESP=FF(JP)+(FF(JPADJ)-FF(JP))*(PSI-PPSI(IDAT,JP))/
     *              (PPSI(IDAT,JPADJ)-PPSI(IDAT,JP))

        RESP=E*FK*(RFACT*RFACT)*DEXP(RESP)
        RETURN
      END



      SUBROUTINE READGS(NGAU,X,W)
C*************************************************************
C  Routine returns data needed for Gaussian quadrature
C
C  Input:
C    NGAU = Order of quadrature (4, 8, 16, or 32)
C  Return:
C      X(NGAU), W(NGAU) : Quadrature ordinates and weights
C*************************************************************
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION X(32),W(32),GAUS(30,2)

      DATA (GAUS(I,1),I=1,2)/ .861136311594053, .339981043584856 /
      DATA (GAUS(I,2),I=1,2)/ .347854845137454, .652145154862546 /
      DATA (GAUS(I,1),I=3,6)/ .960289856497536, .796666477413627,
     1                        .525532409916329, .183434642495650/
      DATA (GAUS(I,2),I=3,6)/ .101228536290376, .222381034453374,
     1                        .313706645877887, .362683783378362 /
      DATA(GAUS(I,1),I=7,14)/ .095012509837637, .281603550779259,
     1      .458016777657227, .617876244402644, .755404408355003,
     2      .865631202387832, .944575023073233, .989400934991649/
      DATA(GAUS(I,2),I=7,14)/ .189450610455068, .182603415044924,
     1      .169156519395003, .149595988816577, .124628971255534,
     2      .095158511682493, .062253523938648, .027152459411754/
      DATA(GAUS(I,1),I=15,30)/.997263861849482, .985611511545268,
     1                        .964762255587506, .934906075937740,
     2                        .896321155766052, .849367613732570,
     3                        .794483795967942, .732182118740290,
     4                        .663044266930215, .587715757240762,
     5                        .506899908932229, .421351276130635,
     6                        .331868602282128, .239287362252137,
     7                        .144471961584796, .048307665687738/
      DATA(GAUS(I,2),I=15,30)/.007018610009470, .016274394730906,
     1                        .025392065309262, .034273862913021,
     2                        .042835898022227, .050998059262376,
     3                        .058684093478536, .065822222776362,
     4                        .072345794108849, .078193895787070,
     5                        .083311924226947, .087652093004404,
     6                        .091173878695763, .093844399080805,
     7                        .095638720079275, .096540088514728 /
C
C*** Read in quadrature ordinates and weights and use symmetry
      IF (NGAU.EQ.4) THEN
         DO 5 I = 1,2
           X(5-I) = GAUS(I,1)
5        W(5-I) = GAUS(I,2)
         DO 10 I = 1, 2
           X(I) = -X(5-I)
10       W(I) = W(5-I)
      ELSEIF (NGAU.EQ.8) THEN
         DO 20 I = 1,4
           X(9-I) = GAUS(I+2,1)
20       W(9-I) = GAUS(I+2,2)
         DO 30 I = 1 , 4
           X(I) = -X(9-I)
30         W(I) = W(9-I)
      ELSEIF(NGAU.EQ.16)THEN
         DO 40 I = 1,8
           X(8+I) = GAUS(I+6,1)
40       W(8+I) = GAUS(I+6,2)
         DO 50 I = 9, 16
           X(17-I) = -X(I)
50       W(17-I) = W(I)
      ELSE
         DO 60 I = 1,16
           X(33-I) = GAUS(I+14,1)
 60      W(33-I) = GAUS(I+14,2)
         DO 70 I = 1,16
           X(I) = -X(33-I)
70       W(I) = W(33-I)
       ENDIF
       RETURN
       END


      DOUBLE PRECISION FUNCTION GMU(EE,MM,NN)
C   DOUBLE PRECISION VERSION
C
C   COMPUTES MASS ATTENUATION COEFFICIENTS AND MASS KERMA COEFFICIENTS
C   FOR AIR, WATER, CONCRETE, IRON, AND LEAD
C   OVER THE RANGE 0.01 TO 10 MEV
C
C   Reference:  Hubbell, J. H., "Photon Mass Attenuation and
C               Energy-Absorption Coefficients from 1 keV to 20 MeV,"
C               Int. J. Appl. Radiat. Isot., 33, 1269-1290, 1982.
C
C   INDEX NN DETERMINES MATERIAL:   1. AIR
C                                   2. WATER
C                                   3. CONCRETE
C                                   4. IRON
C                                   5. LEAD
C
C   INDEX MM DETERMINES TYPE GMU:   1. TOTAL MASS INTER. COEFFICIENT
C                                   2. MASS KERMA COEFFICIENT
C
C   VARIABLE EE IS THE ENERGY (MeV)
C
C   FUNCTION RETURNS COEFFICIENT (CM^2/G)
C ---------------------------------------------------------------------
      IMPLICIT REAL*8(A-H,O-Z)
      DIMENSION N(5),E(33,5),COEF1(33,5),COEF2(33,5)
C  NUMBER OF ELEMENTS IN ENERGY RANGE
      DATA N/4*25,33/
C  ENERGY GROUP STRUCTURE
      DATA E/.01,.015,.02,.03,.04,.05,.06,.08,.10,.15,.2,.3,.4
     &,.5,.6,.8,1.,1.5,2.,3.,4.,5.,6.,8.,10.,8*0.
     &,.01,.015,.02,.03,.04,.05,.06,.08,.10,.15,.2,.3,.4
     &,.5,.6,.8,1.,1.5,2.,3.,4.,5.,6.,8.,10.,8*0.
     &,.01,.015,.02,.03,.04,.05,.06,.08,.10,.15,.2,.3,.4
     &,.5,.6,.8,1.,1.5,2.,3.,4.,5.,6.,8.,10.,8*0.
     &,.01,.015,.02,.03,.04,.05,.06,.08,.10,.15,.2,.3,.4
     &,.5,.6,.8,1.,1.5,2.,3.,4.,5.,6.,8.,10.,8*0.
     &,.01,.01304,.01304,.015,.0152,.0152,.015861,.015861,.02,.03,.04
     &,.05,.06,.08,.088004,.088004,.10,.15,.2,.3,.4,.5
     &,.6,.8,1.,1.5,2.,3.,4.,5.,6.,8.,10./
      DATA ((COEF1(I,J),I=1,33),J=1,1)/
     & 5.016D+00,1.581D+00,7.643D-01,3.501D-01,2.471D-01
     &,2.073D-01,1.871D-01,1.661D-01,1.541D-01,1.356D-01,1.234D-01
     &,1.068D-01,9.548D-02,8.712D-02,8.056D-02,7.075D-02,6.359D-02
     &,5.176D-02,4.447D-02,3.581D-02,3.079D-02,2.751D-02,2.523D-02
     &,2.225D-02,2.045D-02,8*0.D0/
      DATA ((COEF1(I,J),I=1,33),J=2,2)/
     & 5.223D+00,1.639D+00,7.958D-01,3.718D-01,2.668D-01,2.262D-01
     &,2.055D-01,1.835D-01,1.707D-01,1.504D-01,1.370D-01,1.187D-01
     &,1.061D-01,9.687D-02,8.957D-02,7.866D-02,7.070D-02,5.755D-02
     &,4.940D-02,3.969D-02,3.403D-02,3.031D-02,2.771D-02,2.429D-02
     &,2.219D-02,8*0.D0/
      DATA ((COEF1(I,J),I=1,33),J=3,3)/
     & 2.619D+01,8.185D+00,3.605D+00,1.202D+00,6.070D-01,3.918D-01
     &,2.943D-01,2.119D-01,1.781D-01,1.433D-01,1.270D-01,1.082D-01
     &,9.629D-02,8.767D-02,8.098D-02,7.103D-02,6.381D-02,5.197D-02
     &,4.482D-02,3.654D-02,3.189D-02,2.895D-02,2.696D-02,2.450D-02
     &,2.311D-02,8*0.D0/
      DATA ((COEF1(I,J),I=1,33),J=4,4)/
     & 1.690D+02,5.656D+01,2.546D+01,8.109D+00,3.601D+00,1.944D+00
     &,1.197D+00,5.918D-01,3.701D-01,1.960D-01,1.458D-01,1.098D-01
     &,9.398D-02,8.413D-02,7.703D-02,6.698D-02,5.994D-02,4.883D-02
     &,4.265D-02,3.622D-02,3.311D-02,3.146D-02,3.057D-02,2.991D-02
     &,2.994D-02,8*0.D0/
      DATA ((COEF1(I,J),I=1,33),J=5,5)/
     & 1.306D+02,6.100D+01,1.580D+02,1.116D+02,1.070D+02,1.480D+02
     &,1.320D+02,1.530D+02,8.636D+01,3.032D+01,1.436D+01,8.041D+00
     &,5.020D+00,2.419D+00,1.610D+00,7.380D+00,5.550D+00,2.014D+00
     &,9.985D-01,4.026D-01,2.323D-01,1.613D-01,1.248D-01,8.869D-02
     &,7.103D-02,5.222D-02,4.607D-02,4.234D-02,4.197D-02,4.272D-02
     &,4.391D-02,4.675D-02,4.972D-02/
      DATA ((COEF2(I,J),I=1,33),J=1,1)/
     & 4.640D+00,1.300D+00,5.255D-01,1.501D-01,6.694D-02
     &,4.031D-02,3.004D-02,2.393D-02,2.318D-02,2.494D-02,2.672D-02
     &,2.872D-02,2.949D-02,2.966D-02,2.953D-02,2.882D-02,2.787D-02
     &,2.545D-02,2.342D-02,2.054D-02,1.866D-02,1.737D-02,1.644D-02
     &,1.521D-02,1.446D-02,8*0.D0/
      DATA ((COEF2(I,J),I=1,33),J=2,2)/
     & 4.840D+00,1.340D+00,5.367D-01,1.520D-01,6.803D-02,4.155D-02
     &,3.152D-02,2.583D-02,2.539D-02,2.762D-02,2.966D-02,3.192D-02
     &,3.279D-02,3.299D-02,3.284D-02,3.205D-02,3.100D-02,2.831D-02
     &,2.604D-02,2.278D-02,2.063D-02,1.913D-02,1.804D-02,1.657D-02
     &,1.566D-02,8*0.D0/
      DATA ((COEF2(I,J),I=1,33),J=3,3)/
     & 2.467D+01,7.582D+00,3.217D+00,9.454D-01,3.959D-01,2.048D-01
     &,1.230D-01,6.154D-02,4.180D-02,3.014D-02,2.887D-02,2.937D-02
     &,2.980D-02,2.984D-02,2.964D-02,2.887D-02,2.790D-02,2.554D-02
     &,2.348D-02,2.086D-02,1.929D-02,1.828D-02,1.760D-02,1.680D-02
     &,1.639D-02,8*0.D0/
      DATA ((COEF2(I,J),I=1,33),J=4,4)/
     & 1.367D+02,4.895D+01,2.257D+01,7.237D+00,3.146D+00,1.630D+00
     &,9.538D-01,4.093D-01,2.181D-01,7.970D-02,4.840D-02,3.374D-02
     &,3.050D-02,2.922D-02,2.843D-02,2.718D-02,2.604D-02,2.358D-02
     &,2.195D-02,2.036D-02,1.984D-02,1.976D-02,1.991D-02,2.043D-02
     &,2.100D-02,8*0.D0/
      DATA ((COEF2(I,J),I=1,33),J=5,5)/
     & 1.256D+02,6.100D+01,1.130D+02,8.939D+01,8.080D+01,1.090D+02
     &,9.880D+01,1.140E+02,6.923D+01,2.550D+01,1.221D+01,6.796D+00
     &,4.177D+00,1.936D+00,1.450D+00,2.310D+00,2.229D+00,1.135D+00
     &,6.229D-01,2.581D-01,1.439D-01,9.564D-02,7.132D-02,4.838D-02
     &,3.787D-02,2.714D-02,2.407D-02,2.351D-02,2.463D-02,2.600D-02
     &,2.730D-02,2.948D-02,3.114D-02/
C
C     PERFORM INTERPOLATION
C
      DO 101 I = 1,N(NN)
      IF (EE.EQ.E(I,NN)) GO TO 102
      IF (EE.LT.E(I,NN)) GO TO 103
  101 CONTINUE
  102 IF(MM.EQ.1) GMU = COEF1(I,NN)
      IF(MM.EQ.2) GMU = COEF2(I,NN)
      GO TO 104
  103 II = I-1
      GOTO (201,202), MM
  201 YY1 = DLOG(COEF1(II,NN))
      YY2 = DLOG(COEF1(I,NN))
      GOTO 203
  202 YY1 = DLOG(COEF2(II,NN))
      YY2 = DLOG(COEF2(I,NN))
  203 XX1 = DLOG(E(II,NN))
      XX2 = DLOG(E(I,NN))
      GG  = DLOG(EE)
      ZZ  = YY1 + (GG-XX1)*(YY2-YY1)/(XX2-XX1)
      GMU = DEXP(ZZ)
 104  CONTINUE
      RETURN
      END


      DOUBLE PRECISION FUNCTION BUGP(N,E,XX,IERR)
C
C THIS FUNCTION SUBPROGRAM EVALUATES GAMMA RAY EXPOSURE BUILDUP FACTORS
C USING THE GEOMETRIC PROGRESSION FITTING FUNCTION AND DATA AS REPORTED
C IN HARIMA, ET AL., "VALIDITY OF THE GEOMETRIC PROGRESSION GAMMA-RAY
C BUILDUP FACTORS," NUCL. SCI. ENG. 94, SEPT. 1986.  VARIATION OF
C COEFFICIENTS WITH ENERGY IS DETERMINED BY LINEAR INTERPOLATION IN
C LOG(E).  PARTS HAVE BEEN TAKEN FROM THE SUBROUTINE "SETP" USED IN
C THE CODE "QAD-CGGP", WRITTEN BY TRUBEY (RSIC, OAK RIDGE NATIONAL
C LABS).
C
C ARGUMENT N IS THE MATERIAL INDEX:  1. AIR
C                                    2. WATER
C                                    3. CONCRETE
C                                    4. IRON
C                                    5. LEAD
C ARGUMENT E IS THE ENERGY E (MEV)
C ARGUMENT XX IS THE NUMBER OF MEAN FREE PATHS
C
      IMPLICIT REAL*8(A-H,O-Z)
      DIMENSION EE(25),BFSET(5,25,5),FLOGE(25)
C     MEANING OF INDICES: BFSET(MATERIAL,E,COEFFICIENT)
      COMMON/BLD01/PARAM(5)
C
C data for air
      DATA ((BFSET(1,J,K),K=1,5),J=1,25)/1.170,
     & 0.459,0.175,13.73,-0.0862, 1.407,0.512,0.161,14.40,-0.0819,
     &2.292,0.693,0.102,13.34,-0.0484, 3.390,1.052,-0.004,19.76,-0.0068,
     & 4.322,1.383,-0.071,13.51,0.0270, 4.837,1.653,-0.115,13.66,0.0511,
     & 4.929,1.983,-0.159,13.74,0.0730, 4.580,2.146,-0.178,12.83,0.0759,
     & 3.894,2.148,-0.173,14.46,0.0698, 3.345,2.147,-0.176,14.08,0.0719,
     & 2.887,1.990,-0.160,14.13,0.0633, 2.635,1.860,-0.146,14.24,0.0583,
     & 2.496,1.736,-0.130,14.32,0.0505, 2.371,1.656,-0.120,14.27,0.0472,
     & 2.207,1.532,-0.103,14.12,0.0425, 2.102,1.428,-0.086,14.35,0.0344,
     & 1.939,1.265,-0.057,14.24,0.0232, 1.835,1.173,-0.039,14.07,0.0161,
     & 1.712,1.051,-0.011,13.67,0.0024, 1.627,0.983,0.006,13.51,-0.0051,
     & 1.558,0.943,0.017,13.82,-0.0117, 1.505,0.915,0.025,16.37,-0.0231,
     & 1.418,0.891,0.032,12.06,-0.0167, 1.358,0.875,0.037,14.01,-0.0226,
     & 1.267,0.844,0.048,14.55,-0.0344/

C data for water
      DATA ((BFSET(2,J,K),K=1,5),J=1,25)/1.182,
     & 0.463,0.175,14.23,-0.0908, 1.427,0.549,0.143,14.86,-0.0707,
     & 2.335,0.736,0.087,13.28,-0.0419, 3.477,1.117,-0.019,11.67,0.0026,
     & 4.461,1.457,-0.084,13.62,0.0341, 4.983,1.730,-0.126,13.64,0.0561,
     & 5.059,2.059,-0.168,13.67,0.0770, 4.663,2.221,-0.186,13.33,0.0826,
     & 3.897,2.242,-0.185,14.19,0.0777, 3.478,2.154,-0.176,14.50,0.0774,
     & 2.920,2.022,-0.164,14.21,0.0655, 2.660,1.882,-0.149,14.24,0.0595,
     & 2.500,1.766,-0.135,14.33,0.0546, 2.377,1.679,-0.124,14.23,0.0503,
     & 2.212,1.544,-0.105,14.36,0.0437, 2.103,1.441,-0.089,14.22,0.0378,
     & 1.939,1.269,-0.058,14.52,0.0246, 1.839,1.173,-0.039,14.07,0.0161,
     & 1.710,1.056,-0.013,11.82,0.0047, 1.621,0.989,0.004,13.45,-0.0041,
     & 1.554,0.939,0.018,13.55,-0.0122, 1.507,0.903,0.029,16.13,-0.0272,
     & 1.422,0.879,0.035,13.36,-0.0191, 1.362,0.859,0.042,13.37,-0.0247,
     & 1.267,0.843,0.047,15.08,-0.0336/

C data for concrete
      DATA ((BFSET(3,J,K),K=1,5),J=1,25)/1.029,
     & 0.364,0.240,14.12,-0.1704, 1.067,0.389,0.214,12.68,-0.1126,
     & 1.212,0.421,0.201,14.12,-0.1079, 1.455,0.493,0.171,14.53,-0.0925,
     & 1.737,0.628,0.115,15.82,-0.0600, 2.125,0.664,0.118,11.90,-0.0615,
     & 2.557,0.895,0.042,14.37,-0.0413, 2.766,1.069,0.001,12.64,-0.0251,
     & 2.824,1.315,-0.049,8.66,-0.0048, 2.716,1.430,-0.070,18.52,0.0108,
     & 2.522,1.492,-0.082,16.59,0.0161, 2.372,1.494,-0.085,15.96,0.0194,
     & 2.271,1.466,-0.082,16.25,0.0195, 2.192,1.434,-0.078,17.02,0.0199,
     & 2.066,1.386,-0.073,15.07,0.0202, 1.982,1.332,-0.065,15.38,0.0193,
     & 1.848,1.227,-0.047,16.41,0.0160, 1.775,1.154,-0.033,14.35,0.0100,
     &1.671,1.054,-0.010,10.47,-0.0008, 1.597,0.988,0.008,12.53,-0.0115,
     & 1.527,0.951,0.020,9.99,-0.0184, 1.478,0.940,0.021,13.11,-0.0163,
     & 1.395,0.917,0.028,13.45,-0.0213, 1.334,0.901,0.035,12.56,-0.0267,
     & 1.260,0.823,0.065,14.28,-0.0581/

C data for iron
      DATA ((BFSET(4,J,K),K=1,5),J=1,25)/1.004,
     & 1.583,-0.565,5.53,0.3568, 1.012,0.130,0.620,11.39,-0.6162,
     & 1.028,0.374,0.190,29.34,-0.3170, 1.058,0.336,0.248,11.65,-0.1188,
     & 1.099,0.366,0.232,14.01,-0.1354, 1.148,0.405,0.208,14.17,-0.1142,
     & 1.267,0.470,0.180,14.48,-0.0974, 1.389,0.557,0.144,14.11,-0.0791,
     & 1.660,0.743,0.079,14.12,-0.0476, 1.839,0.911,0.034,13.23,-0.0334,
     &1.973,1.095,-0.009,11.86,-0.0183,1.992,1.187,-0.027,10.72,-0.0140,
     & 1.974,1.230,-0.036,9.30,-0.0110, 1.942,1.251,-0.041,7.89,-0.0090,
     & 1.892,1.244,-0.040,6.95,-0.0123, 1.846,1.223,-0.037,6.74,-0.0131,
     & 1.750,1.197,-0.040,15.90,0.0110, 1.712,1.126,-0.022,7.34,-0.0047,
     &1.627,1.059,-0.005,11.99,-0.0132, 1.553,1.026,0.005,12.93,-0.0191,
     & 1.483,1.009,0.012,13.12,-0.0258, 1.442,0.980,0.023,13.37,-0.0355,
     & 1.354,0.974,0.029,13.65,-0.0424, 1.297,0.949,0.042,13.97,-0.0561,
     & 1.194,1.048,-0.002,5.01,0.0584/

C data for lead
      DATA ((BFSET(5,J,K),K=1,5),J=1,25)/0.,
     & 0.,0.,0.,0., 0.,0.,0.,0.,0., 1.006,0.230,0.442,12.61,-0.5099,
     & 1.013,0.302,0.331,10.34,-0.3011, 1.024,0.289,0.289,12.38,-0.1453,
     & 1.029,0.423,0.179,17.00,-0.1217, 1.058,0.357,0.238,12.96,-0.1230,
     & 2.165,1.323,0.079,12.13,-0.0731, 1.520,0.337,0.019,9.05,-0.0108,
     & 1.201,0.271,0.171,5.00,0.0842, 1.135,0.523,0.140,17.37,-0.0874,
     & 1.180,0.597,0.113,16.76,-0.0596, 1.233,0.631,0.107,14.62,-0.0533,
     & 1.271,0.684,0.089,14.56,-0.0417, 1.334,0.738,0.073,13.84,-0.0334,
     & 1.372,0.789,0.059,13.44,-0.0288, 1.409,0.865,0.039,13.11,-0.0217,
     & 1.425,0.903,0.036,13.26,-0.0319, 1.383,0.967,0.027,13.51,-0.0385,
     & 1.328,1.009,0.023,14.06,-0.0403, 1.303,0.954,0.052,13.90,-0.0714,
     & 1.233,1.127,-0.012,5.00,0.0459, 1.175,1.149,-0.005,5.26,0.0484,
     & 1.135,1.167,0.002,5.53,0.0488, 1.083,1.190,0.017,6.11,0.0646/

C
C
C * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

C     ** ENERGY INTERPOLATION OF GP PARAMETERS **
      DATA EE /0.015, 0.02, 0.03, 0.04, 0.05, 0.06, 0.08, 0.1,
     *         0.15,  0.2,  0.3,  0.4,  0.5,  0.6,  0.8,  1.0,
     *         1.5,   2.,   3.,   4.,   5.,   6.,   8.,  10.,
     *        15. /
C
C
      IERR = 0
      IF (XX.LE.0.D0) THEN
        BUGP = 1.D0
        RETURN
      END IF
      IF (E.LT..015D0.OR.E.GT.15.D0) THEN
        IERR = 1
        BUGP = 1.D0
        RETURN
      END IF
      IF (N.LT.1.OR.N.GT.5) THEN
        IERR = 2
        BUGP = 1.D0
        RETURN
      END IF
      IF (XX.GT.40.D0) THEN
        IERR = 3
        BUGP = 1.D0
        RETURN
      END IF
C
C
      NLIM=0
    1 NLIM=NLIM+1
        IF(BFSET(N,NLIM,1).GT.0) GO TO 2
      GO TO 1
    2 CONTINUE
C
      DO 3 I=1,25
        FLOGE(I)=DLOG(EE(I))
    3 CONTINUE
C
   10 CONTINUE
C
        NJ=NLIM
   12 CONTINUE
        IF(E.LE.EE(NJ)) GO TO 13
        IF(NJ.GE.25) GO TO 100
        NJ=NJ+1
        GO TO 12
C
   13 CONTINUE
        IF(NJ.LE.NLIM) GO TO 100
        IF(NJ.EQ.NLIM+1.OR.NJ.EQ.25) GO TO 200
        GO TO 300
C
C     ** E<EE(NLIM) OR EE(25)<E **
  100 DO 101 L=1,5
        PARAM(L)=BFSET(N,NJ,L)
  101 CONTINUE
      GO TO 999
C
C     ** EE(NLIM)<E<EE(NLIM+1)
C              OR EE(24)<E<EE(25) **
C          (PARABOLIC INTERPOLATION)
  200 X=DLOG(E)
      IF (NJ.EQ.25) NJ=NJ-1
      XI=FLOGE(NJ-1)
      XJ=FLOGE(NJ)
      XK=FLOGE(NJ+1)
      CI=(X-XJ)/(XI-XJ)*(X-XK)/(XI-XK)
      CJ=(X-XI)/(XJ-XI)*(X-XK)/(XJ-XK)
      CK=(X-XI)/(XK-XI)*(X-XJ)/(XK-XJ)
      DO 201 L=1,5
        PARAM(L)=CI*BFSET(N,NJ-1,L)
     *            +CJ*BFSET(N,NJ,L)
     *            +CK*BFSET(N,NJ+1,L)
 201  CONTINUE
      GO TO 999
C
C     ** EE(NJ-2)<EE(NJ-1)<E<EE(NJ)<EE(NJ+1) **
C          (REVISED PARABOLIC INTERPOLATION)
  300 X=DLOG(E)
      XI=FLOGE(NJ-2)
      XJ=FLOGE(NJ-1)
      XK=FLOGE(NJ)
      XL=FLOGE(NJ+1)
      CI=0.5*(X-XJ)/(XI-XJ)*(X-XK)/(XI-XK)
      CJ=0.5*(X-XI)/(XJ-XI)*(X-XK)/(XJ-XK)
      CK=0.5*(X-XI)/(XK-XI)*(X-XJ)/(XK-XJ)
      DJ=0.5*(X-XK)/(XJ-XK)*(X-XL)/(XJ-XL)
      DK=0.5*(X-XJ)/(XK-XJ)*(X-XL)/(XK-XL)
      DL=0.5*(X-XJ)/(XL-XJ)*(X-XK)/(XL-XK)
      DO 301 L=1,5
        PARAM(L)=CI*BFSET(N,NJ-2,L)
     *            +(CJ+DJ)*BFSET(N,NJ-1,L)
     *            +(CK+DK)*BFSET(N,NJ,L)
     *            +DL*BFSET(N,NJ+1,L)
  301 CONTINUE
C
C   EVALUATE BUILDUP FACTOR
C
  999 BUGP=BLDUP(XX)
      RETURN
      END


C * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

      REAL*8 FUNCTION BLDUP(TTT)
C      ** BUILDUP FACTORS CALCULATED BY GP-METHOD **
      IMPLICIT REAL*8(A-H,O-Z)
       COMMON /BLD01/ PARAM(5)
C         TTT  = MU TIMES DISTANCE
C
      IF(TTT.LE. 0.) GO TO 100
      IF(TTT.GT.40.) GO TO 200
C
      FK=PARAM(2)*DEXP(PARAM(3)*DLOG(TTT))
     *     +PARAM(5)*FUNCF(TTT,PARAM(4))
       IF(FK.LE.0.) GO TO 100
      GO TO 300
C
  200 FK35=PARAM(2)*DEXP(PARAM(3)*DLOG(35.D0))
     *     +PARAM(5)*FUNCF(35.D0,PARAM(4))
      FK40=PARAM(2)*DEXP(PARAM(3)*DLOG(40.D0))
     *     +PARAM(5)*FUNCF(40.D0,PARAM(4))
      TEMP=(1.D0-DEXP(0.1D0*DLOG(TTT/35.D0)))
     * /(1.-DEXP(0.1*DLOG(40.D0/35.D0))) * DLOG((FK40-1.D0)/(FK35-1.D0))
      FK = 1.0D0+(FK35-1.0D0)*DEXP(TEMP)
C
  300 TEMP=TTT
       IF(FK.NE.1.) TEMP=(DEXP(TTT*DLOG(FK))-1.)/(FK-1.)
      BLDUP=1.+(PARAM(1)-1.)*TEMP
      RETURN
C
  100 BLDUP=1.
      RETURN
      END

C * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

      REAL*8 FUNCTION FUNCF(X,XK)
      IMPLICIT REAL*8(A-H,O-Z)
C
      T=X/XK
        IF(T.GE.7.) GO TO 1
      FUNCF=1.-(1.+DEXP(4.D0))/(DEXP(2.*T)+DEXP(4.D0))
      RETURN
    1 FUNCF=1.
      RETURN
      END
